# -*- coding: utf-8 -*-
from django.db import models
from django.conf import settings
from django.utils.translation import ugettext as _

from jmb.core.db.cmsmodels import StatusModel
from cms.models.pluginmodel import CMSPlugin


class PosizioneFromSettings(CMSPlugin, StatusModel):
    template_name = models.CharField(
        _("Template"),
        max_length=100,
        choices=settings.AREARISERVATA_CMSPLUGIN_POSIZIONEFROM_TEMPLATES,
        default="areariservata/cms_plugins/posizione_from.html"
    )
    form = models.CharField(
        _("Form"),
        max_length=100,
        choices=settings.AREARISERVATA_CMSPLUGIN_POSIZIONEFROM_FORM_CLASSES,
        default="acomea.areariservata.forms.PosizioneFromForm",
    )
    form_template_name = models.CharField(
        _("Form template"),
        max_length=100,
        choices=settings.AREARISERVATA_CMSPLUGIN_POSIZIONEFROM_FORM_TEMPLATES,
        default="areariservata/cms_plugins/posizione_from_form.html"
    )
    unsuccess_slug = models.SlugField(
        _("Unsuccess page"),
        help_text=_("slug pagina insuccesso Posizione From")
    )
    fallback_slug = models.SlugField(
        _("Fallback page"),
        blank=True,
        help_text=_("(Opzionale) slug pagina fallback (in caso di anomalie)")
    )

    def __unicode__(self):
        return "%s %s" % (('On - ' if self.status else 'Off - '), self.template_name)

    class Meta:
        app_label = "areariservata"


class PosizioneToSettings(CMSPlugin, StatusModel):
    template_name = models.CharField(
        _("Template"),
        max_length=100,
        choices=settings.AREARISERVATA_CMSPLUGIN_POSIZIONETO_TEMPLATES,
        default="areariservata/cms_plugins/posizione_to.html"
    )
    form = models.CharField(
        _("Form"),
        max_length=100,
        choices=settings.AREARISERVATA_CMSPLUGIN_POSIZIONETO_FORM_CLASSES,
        default="acomea.areariservata.forms.PosizioneToForm",
    )
    form_template_name = models.CharField(
        _("Form template"),
        max_length=100,
        choices=settings.AREARISERVATA_CMSPLUGIN_POSIZIONETO_FORM_TEMPLATES,
        default="areariservata/cms_plugins/posizione_to_form.html"
    )
    unsuccess_slug = models.SlugField(
        _("Unsuccess page"),
        help_text=_("slug pagina insuccesso Posizione To")
    )
    fallback_slug = models.SlugField(
        _("Fallback page"),
        help_text=_("(Opzionale) slug pagina fallback (in caso di anomalie)")
    )

    def __unicode__(self):
        return "%s %s" % (('On - ' if self.status else 'Off - '), self.template_name)

    class Meta:
        app_label = "areariservata"


###################################
###          RIEPILOGO          ###
###################################
class InvestimentoRiepilogoSettings(CMSPlugin, StatusModel):
    template_name = models.CharField(
        _("Template"),
        max_length=100,
        choices=settings.AREARISERVATA_CMSPLUGIN_INVESTIMENTORIEPILOGO_TEMPLATES,
        default="areariservata/cms_plugins/investimento_riepilogo.html"
    )
    form = models.CharField(
        _("Form"),
        max_length=100,
        choices=settings.AREARISERVATA_CMSPLUGIN_INVESTIMENTORIEPILOGO_FORM_CLASSES,
        default="acomea.areariservata.forms.InvestimentoRiepilogoForm",
    )
    form_template_name = models.CharField(
        _("Form template"),
        max_length=100,
        choices=settings.AREARISERVATA_CMSPLUGIN_INVESTIMENTORIEPILOGO_FORM_TEMPLATES,
        default="areariservata/cms_plugins/investimento_riepilogo_form.html"
    )
    unsuccess_slug = models.SlugField(
        _("Unsuccess page"),
        help_text=_("slug pagina insuccesso Investimento riepilogo")
    )
    fallback_slug = models.SlugField(
        _("Fallback page"),
        help_text=_("(Opzionale) slug pagina fallback (in caso di anomalie)")
    )

    def __unicode__(self):
        return "%s %s" % (('On - ' if self.status else 'Off - '), self.template_name)

    class Meta:
        app_label = "areariservata"


class SwitchRiepilogoSettings(CMSPlugin, StatusModel):
    template_name = models.CharField(
        _("Template"),
        max_length=100,
        choices=settings.AREARISERVATA_CMSPLUGIN_SWITCHRIEPILOGO_TEMPLATES,
        default="areariservata/cms_plugins/switch_riepilogo.html"
    )
    form = models.CharField(
        _("Form"),
        max_length=100,
        choices=settings.AREARISERVATA_CMSPLUGIN_SWITCHRIEPILOGO_FORM_CLASSES,
        default="acomea.areariservata.forms.SwitchRiepilogoForm",
    )
    form_template_name = models.CharField(
        _("Form template"),
        max_length=100,
        choices=settings.AREARISERVATA_CMSPLUGIN_SWITCHRIEPILOGO_FORM_TEMPLATES,
        default="areariservata/cms_plugins/switch_riepilogo_form.html"
    )
    unsuccess_slug = models.SlugField(
        _("Unsuccess page"),
        help_text=_("slug pagina insuccesso Switch riepilogo")
    )
    fallback_slug = models.SlugField(
        _("Fallback page"),
        help_text=_("(Opzionale) slug pagina fallback (in caso di anomalie)")
    )

    def __unicode__(self):
        return "%s %s" % (('On - ' if self.status else 'Off - '), self.template_name)

    class Meta:
        app_label = "areariservata"


class DisinvestimentoRiepilogoSettings(CMSPlugin, StatusModel):
    template_name = models.CharField(
        _("Template"),
        max_length=100,
        choices=settings.AREARISERVATA_CMSPLUGIN_DISINVESTIMENTORIEPILOGO_TEMPLATES,
        default="areariservata/cms_plugins/disinvestimento_riepilogo.html"
    )
    form = models.CharField(
        _("Form"),
        max_length=100,
        choices=settings.AREARISERVATA_CMSPLUGIN_DISINVESTIMENTORIEPILOGO_FORM_CLASSES,
        default="acomea.areariservata.forms.DisinvestimentoRiepilogoForm",
    )
    form_template_name = models.CharField(
        _("Form template"),
        max_length=100,
        choices=settings.AREARISERVATA_CMSPLUGIN_DISINVESTIMENTORIEPILOGO_FORM_TEMPLATES,
        default="areariservata/cms_plugins/disinvestimento_riepilogo_form.html"
    )
    unsuccess_slug = models.SlugField(
        _("Unsuccess page"),
        help_text=_("slug pagina insuccesso Disinvestimento riepilogo")
    )
    fallback_slug = models.SlugField(
        _("Fallback page"),
        help_text=_("(Opzionale) slug pagina fallback (in caso di anomalie)")
    )

    def __unicode__(self):
        return "%s %s" % (('On - ' if self.status else 'Off - '), self.template_name)

    class Meta:
        app_label = "areariservata"


###################################
###          REDIRECTS          ###
###################################
class RedirectIsNotAreaActiveSettings(CMSPlugin, StatusModel):
    pageslug = models.SlugField(
        _("Pagina redirect"),
        help_text=_("slug pagina redirect")
    )

    def __unicode__(self):
        return "%s %s" % (('On - ' if self.status else 'Off - '), self.pageslug)

    class Meta:
        app_label = "areariservata"


class RedirectIsNotPersFisicaSettings(CMSPlugin, StatusModel):
    pageslug = models.SlugField(
        _("Pagina redirect"),
        help_text=_("slug pagina redirect")
    )

    def __unicode__(self):
        return "%s %s" % (('On - ' if self.status else 'Off - '), self.pageslug)

    class Meta:
        app_label = "areariservata"


class RedirectIsNotClienteDispositivoSettings(CMSPlugin, StatusModel):
    pageslug = models.SlugField(
        _("Pagina redirect"),
        help_text=_("slug pagina redirect")
    )

    def __unicode__(self):
        return "%s %s" % (('On - ' if self.status else 'Off - '), self.pageslug)

    class Meta:
        app_label = "areariservata"


class InputCodiceFiscaleAcomeASettings(CMSPlugin, StatusModel):
    pageslug = models.SlugField(
        _("Pagina redirect"),
        help_text=_("slug pagina redirect")
    )
    form = models.CharField(
        _("Form"),
        max_length=100,
        choices=settings.AREARISERVATA_CMSPLUGIN_INPUTCODICEFISCALEACOMEA_FORM_CLASSES,
        default="acomea.areariservata.forms.CodiceFiscaleAcomeAForm",
    )
    form_template_name = models.CharField(
        _("Form template"),
        max_length=100,
        choices=settings.AREARISERVATA_CMSPLUGIN_INPUTCODICEFISCALEACOMEA_FORM_TEMPLATES,
        default="areariservata/cms_plugins/codice_fiscale_acomea.html"
    )

    def __unicode__(self):
        return "%s %s" % (('On - ' if self.status else 'Off - '), self.pageslug)

    class Meta:
        app_label = "areariservata"


class OperazioniInCorsoSettings(CMSPlugin, StatusModel):
    template_name = models.CharField(
        _("Template"),
        max_length=100,
        choices=settings.AREARISERVATA_CMSPLUGIN_OPERAZIONI_IN_CORSO_TEMPLATES,
        default="areariservata/cms_plugins/operazioni_in_corso.html"
    )

    def __unicode__(self):
        return "%s %s" % (('On - ' if self.status else 'Off - '), self.template_name)

    class Meta:
        app_label = "areariservata"


class OperazioniEffettuateSettings(CMSPlugin, StatusModel):
    template_name = models.CharField(
        _("Template"),
        max_length=100,
        choices=settings.AREARISERVATA_CMSPLUGIN_OPERAZIONI_EFFETTUATE_TEMPLATES,
        default="areariservata/cms_plugins/operazioni_effettuate.html"
    )

    def __unicode__(self):
        return "%s %s" % (('On - ' if self.status else 'Off - '), self.template_name)

    class Meta:
        app_label = "areariservata"
