# -*- coding: utf-8 -*-
from acomea.profilo.models import RichiestaAttivazioneCliente
from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool
from django.conf import settings
from django.http import HttpResponseRedirect
from django.utils.translation import ugettext as _
from jmb.core.middleware.force_response import ForceResponse
from jmb.core.utils.cmsutils import get_abs_url_from_slug
from jmb.core.utils.functions import import_class

from acomea.areariservata.cms.models import DisinvestimentoRiepilogoSettings, InvestimentoRiepilogoSettings
from acomea.areariservata.cms.models import InputCodiceFiscaleAcomeASettings, OperazioniInCorsoSettings
from acomea.areariservata.cms.models import OperazioniEffettuateSettings, RedirectIsNotClienteDispositivoSettings
from acomea.areariservata.cms.models import PosizioneFromSettings, PosizioneToSettings
from acomea.areariservata.cms.models import RedirectIsNotAreaActiveSettings, RedirectIsNotPersFisicaSettings
from acomea.areariservata.cms.models import SwitchRiepilogoSettings
from acomea.areariservata.models import Operazione
from acomea.areariservata.soap import AcomeaClientRiservataCms

from acomea.areariservata.utils import get_interval


def _reset_position_from_dict(request):
    # questo metodo cancella i dati inseriti in sessione
    # quando si torna indietro
    if 'posizioni_intestatario' in request.session:
        del request.session['posizioni_intestatario']
    if 'posizioni_cointestatario' in request.session:
        del request.session['posizioni_cointestatario']
    if 'posizioni_to' in request.session:
        del request.session['posizioni_to']
    if 'data_operazione' in request.session:
        del request.session['data_operazione']
    if 'result' in request.session:
        del request.session['result']


def _reset_position_to_dict(request):
    if 'posizioni_to' in request.session:
        del request.session['posizioni_to']
    if 'result' in request.session:
        del request.session['result']


def _is_session_empty(request):
    key_list = ['posizioni_intestatario', 'posizioni_cointestatario', 'posizioni_to', 'data_operazione', 'result']
    for key in key_list:
        if key in request.session:
            return False
    return True


class PosizioneFromPlugin(CMSPluginBase):
    module = _('Area riservata')
    model = PosizioneFromSettings
    name = _('Scelta Posizione From')
    admin_preview = False

    def render(self, context, instance, placeholder):
        request = context['request']
        user = request.user
        self.render_template = "profilo/cms_plugins/empty.html"

        if user.is_anonymous() or not instance.status:
            return context

        if not user.is_active:
            return context

        try:
            profilo = user.profilo
            rac = user.profilo.rac
        except:
            return context

        if not profilo.dispositivo and not profilo.is_acomea:
            return context

        self.render_template = instance.form_template_name

        # controllo data_operazione in sessione,
        id_operazione = None
        if 'data_operazione' in request.session:
            if 'id' in request.session['data_operazione']:
                id_operazione = request.session['data_operazione']['id']

        # cerco operazione esistente non completa
        # data_operazione = {}
        operazioni = Operazione.objects.filter(profilo=profilo, id=id_operazione).exclude(status="FN").order_by('-id')
        if operazioni:
            operazione = operazioni[0]
            context.update({
                'object': instance,
                'operazione': operazione,
                'placeholder': placeholder,
            })
            return context

        form_class = import_class(instance.form)
        operazione = Operazione()
        area_riservata = AcomeaClientRiservataCms()
        classe_fondi = None
        try:
            cliente = area_riservata.get_cliente(rac.codice_fiscale)
            classe_fondi = cliente.get("ClasseFondi", None)
        except Exception, e:
            pass

        # modifico i dati dell'operazione se l'utente è AcomeA e vuole simulare il cliente
        if profilo.is_acomea and not 'codice_fiscale' in request.session:
            return context

        if profilo.is_acomea and 'codice_fiscale' in request.session:
            codice_fiscale = request.session['codice_fiscale']
            operazione.set_cliente_from_codice_fiscale(profilo=profilo, codice_fiscale=codice_fiscale)
            operazione.simulazione = True
        else:
            operazione.set_cliente_from_profile(profilo)
            operazione.simulazione = False

        if request.method == 'POST' and 'posizione_from_form_submit' in request.POST:
            posizione_from_form = form_class(request.POST, session=request.session)
            posizioni_intestatario = request.session['posizioni_intestatario']
            posizioni_cointestatario = request.session['posizioni_cointestatario']

            if posizione_from_form.is_valid():

                # intestatario
                if posizione_from_form.cleaned_data.get('posizione_from_int'):
                    operazione.set_intestatario_position_from(
                        int(posizione_from_form.cleaned_data['posizione_from_int']),
                        posizione_from_form.cleaned_data['tipo_operazione']
                    )

                # cointestatario
                if posizione_from_form.cleaned_data.get('posizione_from_coint'):
                    operazione.set_cointestatario_position_from(
                        int(posizione_from_form.cleaned_data['posizione_from_coint']),
                        posizione_from_form.cleaned_data['tipo_operazione']
                    )
                operazione.save()
                # aggiorno sessione
                data_operazione = Operazione.objects.filter(id=operazione.id).values()[0]
                request.session['data_operazione'] = data_operazione
        else:
            _reset_position_from_dict(request)
            data_operazione = {}
            posizioni_intestatario = operazione.get_intestatario_positions_from()
            posizioni_cointestatario = operazione.get_cointestatario_positions_from()
            request.session['posizioni_intestatario'] = posizioni_intestatario
            request.session['posizioni_cointestatario'] = posizioni_cointestatario
            request.session['data_operazione'] = data_operazione
            posizione_from_form = form_class(session=request.session)

        context.update({
            'object': instance,
            'operazione': operazione,
            'classe_fondi': classe_fondi,
            'posizioni_intestatario': posizioni_intestatario,
            'posizioni_cointestatario': posizioni_cointestatario,
            'posizione_from_form': posizione_from_form,
            'placeholder': placeholder,
        })
        return context


plugin_pool.register_plugin(PosizioneFromPlugin)


class PosizioneToPlugin(CMSPluginBase):
    module = _('Area riservata')
    model = PosizioneToSettings
    name = _('Scelta Posizione To')
    admin_preview = False

    def render(self, context, instance, placeholder):
        request = context['request']
        user = request.user
        self.render_template = "profilo/cms_plugins/empty.html"

        if user.is_anonymous() or not instance.status:
            return context

        if not user.is_active:
            return context

        try:
            profilo = user.profilo
            rac = user.profilo.rac
        except:
            return context

        if not profilo.dispositivo and not profilo.is_acomea:
            return context

        # controllo data_operazione in sessione

        id_operazione = None
        if 'data_operazione' in request.session:
            if 'id' in request.session['data_operazione']:
                id_operazione = request.session['data_operazione']['id']
        operazioni = Operazione.objects.filter(
            profilo=profilo, status="IN", id=id_operazione
        ).order_by('-id')

        posizioni_to = []
        if not operazioni:
            return context

        operazione = operazioni[0]
        error_message = None

        self.render_template = instance.form_template_name
        form_class = import_class(instance.form)

        # reset posizione
        if request.method == 'POST' and 'posizione_to_form_reset' in request.POST:
            operazione.reset_position_from()
            _reset_position_from_dict(request)
            raise ForceResponse(HttpResponseRedirect(request.META['PATH_INFO']))

        if request.method == 'POST' and 'posizione_to_form_submit' in request.POST:

            posizione_to_form = form_class(request.POST, session=request.session)

            if posizione_to_form.is_valid():
                tipo_operazione = operazione.tipo_operazione

                # INVESTIMENTO #
                if tipo_operazione == "I":
                    position = operazione.set_sottoscrizione_position_to(
                        int(posizione_to_form.cleaned_data['posizione_to'])
                    )
                    numero_rate_versate = None

                    if posizione_to_form.cleaned_data['numero_rate_versate']:
                        importo = posizione_to_form.cleaned_data['importo_pac']
                        numero_rate_versate = posizione_to_form.cleaned_data['numero_rate_versate']
                    else:
                        importo = posizione_to_form.cleaned_data['importo_pic']

                    operazione.set_valori_sottoscrizione(
                        importo, numero_rate_versate
                    )

                # SWITCH #
                elif tipo_operazione == "S":
                    position = operazione.set_switch_position_to(
                        int(posizione_to_form.cleaned_data['posizione_to'])
                    )
                    quote = numero_rate_versate = None

                    tipo_qta_movimento = posizione_to_form.cleaned_data['tipo_qta_movimento']
                    if posizione_to_form.cleaned_data['numero_rate_versate']:
                        importo = posizione_to_form.cleaned_data['importo_pac']
                        numero_rate_versate = posizione_to_form.cleaned_data['numero_rate_versate']
                    else:
                        importo = posizione_to_form.cleaned_data['importo_pic']
                        quote = posizione_to_form.cleaned_data['quote']
                    operazione.set_valori_switch(
                        tipo_qta_movimento, importo, quote, numero_rate_versate
                    )

                # DISINVESTIMENTO #
                elif tipo_operazione == "D":
                    position = operazione.set_rimborso_position_to()

                    tipo_qta_movimento = posizione_to_form.cleaned_data['tipo_qta_movimento']
                    iban = posizione_to_form.cleaned_data['iban']
                    importo = posizione_to_form.cleaned_data['importo']
                    quote = posizione_to_form.cleaned_data['quote']
                    operazione.set_valori_rimborso(tipo_qta_movimento, iban, importo, quote)

                else:
                    return context

                # mifid checkato, imposto operazione.mifid a True
                if 'mifid' in request.session['data_operazione']:
                    if request.session['data_operazione']['mifid']:
                        operazione.mifid = True

                operazione.save()
                # aggiorno i dati nella request
                data_operazione = Operazione.objects.filter(id=operazione.id).values()[0]
                request.session['data_operazione'] = data_operazione
            else:
                posizioni_to = request.session['posizioni_to']
                request.session['result'] = None

        else:
            _reset_position_to_dict(request)
            posizioni_to = operazione.get_positions_to()
            request.session['posizioni_to'] = posizioni_to
            request.session['result'] = None
            posizione_to_form = form_class(session=request.session)

        context.update({
            'object': instance,
            'operazione': operazione,
            'posizioni_to': posizioni_to,
            'posizione_to_form': posizione_to_form,
            'error_message': error_message,
            'placeholder': placeholder,
        })
        return context


plugin_pool.register_plugin(PosizioneToPlugin)


class InvestimentoRiepilogoPlugin(CMSPluginBase):
    module = _('Area riservata')
    model = InvestimentoRiepilogoSettings
    name = _('Investimento Riepilogo')
    admin_preview = False

    def render(self, context, instance, placeholder):
        request = context['request']
        user = request.user
        self.render_template = "profilo/cms_plugins/empty.html"
        error_message = None

        if user.is_anonymous() or not instance.status:
            return context

        if not user.is_active:
            return context

        try:
            profilo = user.profilo
            rac = user.profilo.rac
        except:
            return context

        if not profilo.dispositivo and not profilo.is_acomea:
            return context

        self.render_template = instance.form_template_name
        form_class = import_class(instance.form)

        # controllo data_operazione in sessione
        id_operazione = None
        if 'data_operazione' in request.session:
            if 'id' in request.session['data_operazione']:
                id_operazione = request.session['data_operazione']['id']

        operazioni = Operazione.objects.filter(
            profilo=profilo, status="FF", tipo_operazione="I", id=id_operazione
        ).order_by('-id')
        if not operazioni:
            return context

        operazione = operazioni[0]

        # reset posizione
        if request.method == 'POST' and 'conferma_investimento_form_reset' in request.POST:
            operazione.reset_position_to()
            _reset_position_to_dict(request)
            operazione.save()
            # _clean_session(request)
            raise ForceResponse(HttpResponseRedirect(request.META['PATH_INFO']))

        elif request.method == 'POST' and 'conferma_investimento_form_submit' in request.POST:
            conferma_investimento_form = form_class(request.POST, session=request.session)
            if conferma_investimento_form.is_valid():
                tipo_operazione = operazione.tipo_operazione
                result = operazione.save_sottoscrizione()
                operazione.save()
                # _clean_session(request)
        else:
            conferma_investimento_form = form_class(session=request.session)

        context.update({
            'object': instance,
            'placeholder': placeholder,
            'operazione': operazione,
            'error_message': error_message,
            'conferma_investimento_form': conferma_investimento_form
        })
        return context


plugin_pool.register_plugin(InvestimentoRiepilogoPlugin)


class SwitchRiepilogoPlugin(CMSPluginBase):
    module = _('Area riservata')
    model = SwitchRiepilogoSettings
    name = _('Switch Riepilogo')
    admin_preview = False

    def render(self, context, instance, placeholder):
        request = context['request']
        user = request.user
        self.render_template = "profilo/cms_plugins/empty.html"
        error_message = None

        if user.is_anonymous() or not instance.status:
            return context

        if not user.is_active:
            return context

        try:
            profilo = user.profilo
            rac = user.profilo.rac
        except:
            return context

        if not profilo.dispositivo and not profilo.is_acomea:
            return context

        self.render_template = instance.form_template_name
        form_class = import_class(instance.form)

        # controllo data_operazione in sessione
        id_operazione = None
        if 'data_operazione' in request.session:
            if 'id' in request.session['data_operazione']:
                id_operazione = request.session['data_operazione']['id']

        operazioni = Operazione.objects.filter(
            profilo=profilo, status="FF", tipo_operazione="S", id=id_operazione
        ).order_by('-id')

        if not operazioni:
            return context

        operazione = operazioni[0]

        # reset posizione
        if request.method == 'POST' and 'conferma_switch_form_reset' in request.POST:
            operazione.reset_position_to()
            _reset_position_to_dict(request)
            operazione.save()
            # _clean_session(request)
            raise ForceResponse(HttpResponseRedirect(request.META['PATH_INFO']))

        if request.method == 'POST' and 'conferma_switch_form_submit' in request.POST:
            conferma_switch_form = form_class(request.POST, session=request.session)
            if conferma_switch_form.is_valid():
                tipo_operazione = operazione.tipo_operazione
                result = operazione.save_switch()
                operazione.save()

        else:
            conferma_switch_form = form_class(session=request.session)

        context.update({
            'object': instance,
            'placeholder': placeholder,
            'operazione': operazione,
            'error_message': error_message,
            'conferma_switch_form': conferma_switch_form
        })
        return context


plugin_pool.register_plugin(SwitchRiepilogoPlugin)


class DisinvestimentoRiepilogoPlugin(CMSPluginBase):
    module = _('Area riservata')
    model = DisinvestimentoRiepilogoSettings
    name = _('Disinvestimento Riepilogo')
    admin_preview = False

    def render(self, context, instance, placeholder):
        request = context['request']
        user = request.user
        self.render_template = "profilo/cms_plugins/empty.html"
        error_message = None

        if user.is_anonymous() or not instance.status:
            return context

        if not user.is_active:
            return context

        try:
            profilo = user.profilo
            rac = user.profilo.rac
        except:
            return context

        if not profilo.dispositivo and not profilo.is_acomea:
            return context

        self.render_template = instance.form_template_name
        form_class = import_class(instance.form)

        # controllo data_operazione in sessione
        id_operazione = None
        if 'data_operazione' in request.session:
            if 'id' in request.session['data_operazione']:
                id_operazione = request.session['data_operazione']['id']

        operazioni = Operazione.objects.filter(
            profilo=profilo, status="FF", tipo_operazione="D", id=id_operazione
        ).order_by('-id')

        if not operazioni:
            return context

        operazione = operazioni[0]

        # reset posizione
        if request.method == 'POST' and 'conferma_disinvestimento_form_reset' in request.POST:
            operazione.reset_position_to()
            _reset_position_to_dict(request)
            operazione.save()
            raise ForceResponse(HttpResponseRedirect(request.META['PATH_INFO']))

        if request.method == 'POST' and 'conferma_disinvestimento_form_submit' in request.POST:
            conferma_disinvestimento_form = form_class(request.POST, session=request.session)
            if conferma_disinvestimento_form.is_valid():
                tipo_operazione = operazione.tipo_operazione
                result = operazione.save_rimborso()
                operazione.save()
                # _clean_session(request)
        else:
            conferma_disinvestimento_form = form_class(session=request.session)

        context.update({
            'object': instance,
            'placeholder': placeholder,
            'operazione': operazione,
            'error_message': error_message,
            'conferma_disinvestimento_form': conferma_disinvestimento_form
        })
        return context


plugin_pool.register_plugin(DisinvestimentoRiepilogoPlugin)


class RedirectIsNotAreaActivePlugin(CMSPluginBase):
    model = RedirectIsNotAreaActiveSettings
    name = _("Redirect if Area NOT Active")
    module = _("Area riservata")
    admin_preview = False

    def render(self, context, instance, placeholder):
        request = context['request']
        user = request.user
        self.render_template = "profilo/cms_plugins/empty.html"

        if not instance.status:
            return context

        if not _is_area_active():
            redirect_to = get_abs_url_from_slug(instance.pageslug)
            if redirect_to:
                raise ForceResponse(HttpResponseRedirect(redirect_to))
            else:
                raise ForceResponse(HttpResponseRedirect('/'))
        return context


plugin_pool.register_plugin(RedirectIsNotAreaActivePlugin)


class RedirectIsNotPersFisicaPlugin(CMSPluginBase):
    model = RedirectIsNotPersFisicaSettings
    name = _("Redirect if NOT Persona Fisica")
    module = _("Area riservata")
    admin_preview = False

    def render(self, context, instance, placeholder):
        request = context['request']
        user = request.user
        self.render_template = "profilo/cms_plugins/empty.html"

        if not instance.status:
            return context

        # verifico che l'utente abbia un profilo e una richiesta
        try:
            profilo = user.profilo
            rac = user.profilo.rac
            codice_fiscale = rac.codice_fiscale
        except:
            return context

        if not profilo.is_acomea and not (rac.stato == "9" or rac.stato == "6"):
            raise ForceResponse(HttpResponseRedirect(get_abs_url_from_slug("registrazione")))

        if profilo.is_acomea and not 'codice_fiscale' in request.session:
            return context

        # cliente dispositivo/consultivo da qui in avanti
        if profilo.is_acomea and 'codice_fiscale' in request.session:
            codice_fiscale = request.session['codice_fiscale']

        if not _is_persona_fisica(codice_fiscale):
            redirect_to = get_abs_url_from_slug(instance.pageslug)
            if redirect_to:
                raise ForceResponse(HttpResponseRedirect(redirect_to))
            else:
                raise ForceResponse(HttpResponseRedirect('/'))
        return context


plugin_pool.register_plugin(RedirectIsNotPersFisicaPlugin)


def _is_persona_fisica(codice_fiscale):
    area_riservata = AcomeaClientRiservataCms()
    cliente = area_riservata.get_cliente(codice_fiscale)
    if cliente:
        if cliente['TipoPartecipante'].strip() == "1":
            return True
    return False


def _is_area_active():
    area_riservata = AcomeaClientRiservataCms()
    return area_riservata.is_area_active()


class RedirectIsNotClienteDispositivoPlugin(CMSPluginBase):
    model = RedirectIsNotClienteDispositivoSettings
    name = _("Redirect if NOT Cliente Dispositivo")
    module = _("Area riservata")
    admin_preview = False

    def render(self, context, instance, placeholder):
        request = context['request']
        user = request.user
        self.render_template = "profilo/cms_plugins/empty.html"

        if not instance.status:
            return context

        # verifico che l'utente abbia un profilo e una richiesta
        try:
            profilo = user.profilo
            rac = user.profilo.rac
            codice_fiscale = rac.codice_fiscale
        except:
            return context

        # se profilo acomea, salta controllo
        if profilo.is_acomea:
            return context

        if not _is_cliente_dispositivo(rac):
            redirect_to = get_abs_url_from_slug(instance.pageslug)
            if redirect_to:
                raise ForceResponse(HttpResponseRedirect(redirect_to))
            else:
                raise ForceResponse(HttpResponseRedirect('/'))
        return context


plugin_pool.register_plugin(RedirectIsNotClienteDispositivoPlugin)


def _is_cliente_dispositivo(rac):
    return (rac.stato == '9' and rac.dispositivo())


class InputCodiceFiscaleAcomeAPlugin(CMSPluginBase):
    model = InputCodiceFiscaleAcomeASettings
    name = _("Sidebar Codice Fiscale AcomeA")
    module = _("Area riservata")
    admin_preview = False

    def render(self, context, instance, placeholder):
        request = context['request']
        user = request.user
        self.render_template = "profilo/cms_plugins/empty.html"

        if not instance.status:
            return context

        # verifico che l'utente abbia un profilo e una richiesta
        try:
            profilo = user.profilo
        except:
            return context

        if not profilo.is_acomea:
            return context

        self.render_template = instance.form_template_name
        form_class = import_class(instance.form)
        codice_fiscale = None

        # reset posizione
        if request.method == 'POST' and 'codice_fiscale_acomea_submit' in request.POST:
            codice_fiscale_form = form_class(request.POST)

            if codice_fiscale_form.is_valid():
                codice_fiscale = codice_fiscale_form.cleaned_data['codice_fiscale']
                request.session['codice_fiscale'] = codice_fiscale
                try:
                    rac_simulata = RichiestaAttivazioneCliente.objects.get(codice_fiscale=codice_fiscale)
                    if rac_simulata.stato in ['6', '9']:
                        context.update({
                            'rac_sbloccata': True
                        })
                    else:
                        context.update({
                            'rac_sbloccata': False,
                            'rac_simulata': '{} - {}'.format(rac_simulata.stato, settings.STATI_RAC[rac_simulata.stato])
                        })
                except Exception:
                    context.update({
                        'rac_sbloccata': False
                    })
        else:
            codice_fiscale_form = form_class()
            codice_fiscale = request.session.get('codice_fiscale', None)

        context.update({
            'object': instance,
            'codice_fiscale_form': codice_fiscale_form,
            'codice_fiscale': codice_fiscale,
            'placeholder': placeholder,
        })
        return context


plugin_pool.register_plugin(InputCodiceFiscaleAcomeAPlugin)


class OperazioniInCorsoPlugin(CMSPluginBase):
    module = _('Area riservata')
    model = OperazioniInCorsoSettings
    name = _('Operazioni in Corso')
    admin_preview = False

    def render(self, context, instance, placeholder):
        request = context['request']
        user = request.user
        self.render_template = "profilo/cms_plugins/empty.html"

        if user.is_anonymous() or not instance.status:
            return context

        if not user.is_active:
            return context

        try:
            profilo = user.profilo
            rac = user.profilo.rac
        except:
            return context

        if not profilo.is_cliente and not profilo.is_acomea:
            return context

        codice_fiscale = None
        operazioni_internet = None
        operazioni_da_valorizzare = None
        # modifico i dati dell'operazione se l'utente è AcomeA e vuole simulare il cliente
        if profilo.is_acomea and not 'codice_fiscale' in request.session:
            return context

        if profilo.is_acomea and 'codice_fiscale' in request.session:
            codice_fiscale = request.session['codice_fiscale']

        if profilo.is_cliente:
            codice_fiscale = rac.codice_fiscale

        if not codice_fiscale:
            return context

        area_riservata = AcomeaClientRiservataCms()
        operazioni_internet = area_riservata.get_operazioni_in_corso_web(codice_fiscale)
        operazioni_da_valorizzare = area_riservata.get_operazioni_in_corso_contra(codice_fiscale)
        self.render_template = instance.template_name
        context.update({
            'object': instance,
            'operazioni_internet': operazioni_internet,
            'operazioni_da_valorizzare': operazioni_da_valorizzare,
            'placeholder': placeholder,
        })
        return context


plugin_pool.register_plugin(OperazioniInCorsoPlugin)


class OperazioniEffettuatePlugin(CMSPluginBase):
    module = _('Area riservata')
    model = OperazioniEffettuateSettings
    name = _('Operazioni Effettuate')
    admin_preview = False

    # def get_interval(self, filter):
    #     today = datetime.date.today()
    #     year = today.year
    #     month = today.month
    #     date_end = today
    #
    #     if filter == 'ultimo_mese':
    #         date_begin = today + relativedelta(months=-1)
    #
    #     elif filter == 'ultimi_3_mesi':
    #         date_begin = today + relativedelta(months=-3)
    #
    #     elif filter == 'ultimo_anno':
    #         date_begin = today + relativedelta(years=-1)
    #
    #     else:
    #         date_begin = datetime.date(1970,1,1)
    #
    #     return date_begin, date_end

    def render(self, context, instance, placeholder):
        request = context['request']
        filter = request.GET.get('filter' or None)
        date_begin, date_end = get_interval(filter)

        user = request.user
        self.render_template = "profilo/cms_plugins/empty.html"

        if user.is_anonymous() or not instance.status:
            return context

        if not user.is_active:
            return context

        try:
            profilo = user.profilo
            rac = user.profilo.rac
        except:
            return context

        if not profilo.is_cliente and not profilo.is_acomea:
            return context

        codice_fiscale = None
        operazioni_effettuate = None
        # modifico i dati dell'operazione se l'utente è AcomeA e vuole simulare il cliente
        if profilo.is_acomea and not 'codice_fiscale' in request.session:
            return context

        if profilo.is_acomea and 'codice_fiscale' in request.session:
            codice_fiscale = request.session['codice_fiscale']

        if profilo.is_cliente:
            codice_fiscale = rac.codice_fiscale

        if not codice_fiscale:
            return context

        area_riservata = AcomeaClientRiservataCms()
        operazioni_intestatario = area_riservata.get_operazioni_effettuate(
            codice_fiscale, 'INTESTATARIO', date_begin, date_end
        )
        operazioni_cointestatario = area_riservata.get_operazioni_effettuate(
            codice_fiscale, 'COINTESTATARIO', date_begin, date_end
        )

        self.render_template = instance.template_name
        context.update({
            'object': instance,
            'filter': filter,
            'operazioni_intestatario': operazioni_intestatario,
            'operazioni_cointestatario': operazioni_cointestatario,
            'placeholder': placeholder,
        })
        return context


plugin_pool.register_plugin(OperazioniEffettuatePlugin)


# def _clean_session(request):
#     key_list = ['posizioni_intestatario','posizioni_cointestatario','data_operazione',
#                 'posizioni_to','result']
#     clean_session(request, key_list)


class RenderEcommercePrimasottoscrizionePlugin(CMSPluginBase):
    """
    interroga soap a se non ha posizioni aperte renderizza iframe
    per prima sottoscrizione iniziale
    """
    # import spostato qui per prevenire eventuali dipendenze circolari
    from acomea.profilo.cms.models import PrimaSottoscrizioneSettings

    module = _('Area riservata')
    model = PrimaSottoscrizioneSettings
    name = _('Ecommerce Prima sottoscrizione')
    admin_preview = False

    def render(self, context, instance, placeholder):
        request = context['request']
        user = request.user
        self.render_template = "profilo/cms_plugins/empty.html"
        if user.is_anonymous() or not instance.status:
            return context

        if not user.is_active:
            return context

        try:
            profilo = user.profilo
            rac = user.profilo.rac
        except:
            return context

        if not profilo.is_cliente and not profilo.is_acomea:
            return context

        if profilo.is_acomea and not 'codice_fiscale' in request.session:
            return context

        if profilo.is_acomea and 'codice_fiscale' in request.session:
            codice_fiscale = request.session['codice_fiscale']
        else:
            codice_fiscale = profilo.rac.codice_fiscale

        if _check_is_dead(codice_fiscale):
            self.render_template = instance.template_name
            context.update({
                'object': instance,
                'instance': instance,
                'placeholder': placeholder,
            })
        return context


plugin_pool.register_plugin(RenderEcommercePrimasottoscrizionePlugin)


def _check_is_dead(codice_fiscale, soap_instance=None):
    """
    :param codice_fiscale:
    :param soap_instance:
    :return:
    """
    if not soap_instance:
        soap_instance = AcomeaClientRiservataCms()

    result = soap_instance.get_cliente(codice_fiscale)
    if 'IsDead' in result and result['IsDead']:
        return result['IsDead']
    return False
