#-*- coding: utf-8 -*-

import datetime
import random
import string

from django.conf import settings
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from django.db import models
from django.utils.http import int_to_base36
from django.utils.translation import ugettext_lazy as _
from jmb.core.utils.token import TokenGenerator

from jmb.core.db.models import DateModel, UserModel, StatusModel, OrderedModel, SEOModel
from jmb.i18n.models import TranslationModel, I18NModel
from jmb.core.db.fields import HTMLField, HTMLFieldTinyMCE
from jmb.core.db.models import DateModel, UserModel, StatusModel, OrderedModel

from acomea.areariservata.soap import AcomeaClientRiservataCms
from acomea.profilo.models import AcomeaProfilo, RichiestaAttivazioneCliente


class Operazione(UserModel, DateModel):
    status = models.CharField(
        max_length=3,
        choices=settings.AREARISERVATA_OPERAZIONE_STATUS_CHOICES
    )
    profilo = models.ForeignKey(
        AcomeaProfilo,
        related_name="operazioni",
        verbose_name=_("Profilo")
    )
    codice_cliente = models.IntegerField()
    codice_fiscale = models.CharField(max_length=16)

    tipo_operazione = models.CharField(
        max_length=1,
        choices=settings.AREARISERVATA_OPERAZIONE_TIPO_OPERAZIONE_CHOICES,
    )

    codice_rapporto_from = models.IntegerField(blank=True, null=True)
    codice_deposito_from = models.IntegerField(blank=True, null=True)
    codice_fondo_from = models.CharField(max_length=5, blank=True, null=True)
    descrizione_fondo_from = models.CharField(max_length=255, blank=True, null=True)
    codice_tipocontratto_from = models.CharField(max_length=5, blank=True, null=True)
    tipo_piano_from = models.CharField(max_length=5, blank=True, null=True)
    patrimonio_from = models.DecimalField(default=0, decimal_places=2, max_digits=10)
    quote_from = models.DecimalField(default=0, decimal_places=2, max_digits=10)

    codice_rapporto_to = models.IntegerField(blank=True, null=True)
    codice_deposito_to = models.IntegerField(blank=True, null=True)
    codice_fondo_to = models.CharField(max_length=5, blank=True, null=True)
    descrizione_fondo_to = models.CharField(max_length=255, blank=True, null=True)
    codice_tipocontratto_to = models.CharField(max_length=5, blank=True, null=True)
    tipo_piano_to = models.CharField(max_length=5, blank=True, null=True)

    tipo_qta_movimento = models.CharField(
        max_length=1,
        choices=settings.AREARISERVATA_OPERAZIONE_TIPO_QTA_MOVIMENTO_CHOICES,
        blank=True, null=True
    )

    cognome = models.CharField(max_length=50, blank=True, null=True)
    nome = models.CharField(max_length=50, blank=True, null=True)
    mifid = models.BooleanField(default=False)
    importo = models.DecimalField(default=0, decimal_places=2, max_digits=10)
    numero_rate_piano = models.IntegerField(blank=True, null=True)
    importo_rata = models.DecimalField(default=0, decimal_places=2, max_digits=10)
    numero_rate_versate = models.DecimalField(blank=True, null=True, decimal_places=2, max_digits=10)
    quote = models.DecimalField(blank=True, null=True, decimal_places=3, max_digits=10)  # x switch o rimborso
    iban = models.CharField(max_length=27, blank=True, null=True)

    simulazione = models.BooleanField(default=True)  # utenti AvomeA sempre in 'simulazione'
    numero_operazione = models.IntegerField(blank=True, null=True)
    data_operazione = models.DateTimeField(blank=True, null=True)

    def set_areariservata(self):
        ar = AcomeaClientRiservataCms()
        self.area_riservata = ar

    def set_cliente(self):
        if not hasattr(self, 'area_riservata'):
            self.set_areariservata()
        self.cliente = self.area_riservata.get_cliente(self.codice_fiscale)

    def set_cliente_from_profile(self, profilo):
        try:
            self.codice_fiscale = profilo.rac.codice_fiscale
            self.profilo = profilo
            self.set_codice_cliente()
        except:
            pass

    def set_cliente_from_codice_fiscale(self, profilo, codice_fiscale):
        try:
            self.codice_fiscale = codice_fiscale
            self.profilo = profilo
            self.set_codice_cliente()
        except:
            pass

    def set_codice_cliente(self):
        if not hasattr(self, 'cliente'):
            self.set_cliente()
        self.codice_cliente = self.cliente['CodCliFC']

    def is_area_active(self):
        if not hasattr(self, 'area_riservata'):
            self.set_areariservata()
        return self.area_riservata.is_area_active()

    def is_persona_fisica(self):
        if not hasattr(self, 'cliente'):
            self.set_cliente()
        if self.cliente['TipoPartecipante'].strip() == "1":
            return True
        return False

    def is_rimborso_active(self):
        if not hasattr(self, 'area_riservata'):
            self.set_areariservata()
        return self.area_riservata.op_online_is_rimborso_active()

    def is_sottoscrizione_active(self):
        if not hasattr(self, 'area_riservata'):
            self.set_areariservata()
        return self.area_riservata.op_online_is_sottoscrizione_active()

    def is_switch_active(self):
        if not hasattr(self, 'area_riservata'):
            self.set_areariservata()
        return self.area_riservata.op_online_is_switch_active()

    def get_cliente(self):
        if not hasattr(self, 'cliente'):
            self.set_cliente()
        return self.cliente

    def get_codice_cliente(self):
        if not self.codice_cliente:
            self.set_codice_cliente()
        return self.codice_cliente

    def is_intestatario(self):
        if not hasattr(self, 'cliente'):
            self.set_cliente()
        return self.cliente['IsIntestatario']

    def is_cointestatario(self):
        if not hasattr(self, 'cliente'):
            self.set_cliente()
        return self.cliente['IsCoIntestatario']

    def get_intestatario_positions_from(self):
        """
        restituisce le posizioni intestatario
        """
        if not hasattr(self, 'area_riservata'):
            self.set_areariservata()
        return self.area_riservata.op_online_datagrid(self.codice_cliente, 'INTESTATARIO')

    def get_cointestatario_positions_from(self):
        """
        restituisce le posizioni intestatario
        """
        if not hasattr(self, 'area_riservata'):
            self.set_areariservata()
        return self.area_riservata.op_online_datagrid(self.codice_cliente, 'COINTESTATARIO')

    def must_compile_questionario(self):
        if not hasattr(self, 'area_riservata'):
            self.set_areariservata()
        if self.codice_rapporto_from or self.codice_fondo_from:
            return None
        result = self.area_riservata.qst_must_compile_ar_by_rapp(
            self.codice_fiscale, self.codice_rapporto_from, self.codice_fondo_from
        )
        if (
            result['MCM_Adeguatezza'] or
            result['MCM_Antiriciclaggio'] or
            result['MCM_Appropriatezza']
        ):
            return True
        return False

    def set_intestatario_position_from(self, index, tipo_operazione):
        try:
            position = self.get_intestatario_positions_from()[index]
            self.status = 'IN'
            self.codice_rapporto_from = position['RAPPORTO']
            self.codice_deposito_from = position['NUMCONTRATTO']
            self.codice_fondo_from = position['CODFONDO']
            self.codice_tipocontratto_from = position['CODTC']
            self.descrizione_fondo_from = position['DESCRFONDO']
            self.tipo_piano_from = position['TIPOPIANO']
            self.patrimonio_from = position['PATRIMONIO']
            self.quote_from = position['QUOTE']
            self.tipo_operazione = tipo_operazione
            if self.must_compile_questionario():
                return None
            return position
        except Exception:
            return None

    def set_cointestatario_position_from(self, index, tipo_operazione):
        try:
            position = self.get_cointestatario_positions_from()[index]
            self.status = "IN"
            self.codice_rapporto_from = position['RAPPORTO']
            self.codice_deposito_from = position['NUMCONTRATTO']
            self.codice_fondo_from = position['CODFONDO']
            self.codice_tipocontratto_from = position['CODTC']
            self.descrizione_fondo_from = position['DESCRFONDO']
            self.tipo_piano_from = position['TIPOPIANO']
            self.patrimonio_from = position['PATRIMONIO']
            self.quote_from = position['QUOTE']
            self.tipo_operazione = tipo_operazione
            if self.must_compile_questionario():
                return None
            return position
        except Exception:
            return None

    def reset_position_from(self):
        self.delete()

    def get_positions_to(self):
        if self.tipo_operazione == "I":
            return self.get_sottoscrizione_positions_to()
        elif self.tipo_operazione == "S":
            return self.get_switch_positions_to()
        else:
            return None

    def reset_position_to(self):
        if self.status == "FF":
            self.status = "IN"
            self.codice_rapporto_to = None
            self.codice_deposito_to = None
            self.codice_fondo_to = None
            self.descrizione_fondo_to = None
            self.codice_tipocontratto_to = None
            self.tipo_piano_to = None
            self.numero_rate_piano = None
            self.importo_rata = 0
            self.importo = 0
            self.quote = None
            self.iban = None
            self.numero_rate_versate = None
            self.tipo_qta_movimento = None

            self.cognome = None
            self.nome = None
            self.mifid = False

    def get_sottoscrizione_positions_to(self):
        if not self.tipo_operazione == "I":
            return None

        if not hasattr(self, 'area_riservata'):
            self.set_areariservata()
        return self.area_riservata.op_online_get_elenco_sottoscrizioni_aggiuntive(
            self.codice_cliente, self.codice_rapporto_from,
            self.codice_deposito_from, self.codice_fondo_from,
            self.codice_tipocontratto_from
        )

    def set_sottoscrizione_position_to(self, index):
        if not self.tipo_operazione == "I":
            return None

        try:
            if not hasattr(self, 'cliente'):
                self.set_cliente()
            position = self.get_sottoscrizione_positions_to()[index]

            self.codice_rapporto_to = position['RAPPORTO']
            self.codice_deposito_to = position['NUMCONTRATTO']
            self.codice_fondo_to = position['CODFONDO']
            self.descrizione_fondo_to = position['DESCRFONDO']
            self.codice_tipocontratto_to = position['CODTC']
            self.tipo_piano_to = position['TIPOPIANO']
            self.numero_rate_piano = position['NUMRATEPAC']
            self.importo_rata = position['IMPORTORATA']
            self.cognome = self.cliente['Cognome']
            self.nome = self.cliente['Nome']
            self.mifid = False

            return position
        except Exception, e:
            return e

    def get_switch_positions_to(self):
        if not self.tipo_operazione == "S":
            return None

        if not hasattr(self, 'area_riservata'):
            self.set_areariservata()
        return self.area_riservata.op_online_get_elenco_switch_aggiuntive(
            self.codice_cliente, self.codice_rapporto_from,
            self.codice_deposito_from, self.codice_fondo_from,
            self.codice_tipocontratto_from
        )

    def set_switch_position_to(self, index):
        if not self.tipo_operazione == "S":
            return None

        try:
            if not hasattr(self, 'cliente'):
                self.set_cliente()
            position = self.get_switch_positions_to()[index]

            self.codice_rapporto_to = position['RAPPORTO']
            self.codice_deposito_to = position['NUMCONTRATTO']
            self.codice_fondo_to = position['CODFONDO']
            self.descrizione_fondo_to = position['DESCRFONDO']
            self.codice_tipocontratto_to = position['CODTC']
            self.tipo_piano_to = position['TIPOPIANO']
            self.numero_rate_piano = position['NUMRATEPAC']
            self.importo_rata = position['IMPORTORATA']

            self.cognome = self.cliente['Cognome']
            self.nome = self.cliente['Nome']
            self.mifid = False
            return position
        except Exception, e:
            return e

    def set_rimborso_position_to(self):
        if not self.tipo_operazione == "D":
            return None

        try:
            if not hasattr(self, 'cliente'):
                self.set_cliente()

            self.codice_rapporto_to = self.codice_rapporto_from
            self.codice_deposito_to = self.codice_deposito_from
            self.codice_fondo_to = self.codice_fondo_from
            self.descrizione_fondo_to = self.descrizione_fondo_from
            self.codice_tipocontratto_to = self.codice_tipocontratto_from
            self.numero_rate_versate = 0

            self.cognome = self.cliente['Cognome']
            self.nome = self.cliente['Nome']
            self.mifid = False
            return position
        except Exception, e:
            return e

    def set_valori_sottoscrizione(self, importo, numero_rate_versate=None):
        if not self.tipo_operazione == "I":
            return None

        if self.tipo_piano_to == 'PIC' and importo > 0 and not numero_rate_versate:
            pass
        elif self.tipo_piano_to == 'PAC' and importo > 0 and numero_rate_versate > 0:
            self.numero_rate_versate = numero_rate_versate
        elif self.tipo_piano_to == 'PAD' and importo > 0 and numero_rate_versate > 0:
            self.numero_rate_versate = numero_rate_versate
        else:
            return "ERRORE"  # FIXME: errore

        self.status = 'FF'
        self.importo = importo
        self.tipo_qta_movimento = 'I'

    def set_valori_switch(self, tipo_qta_movimento=None, importo=None, quote=None, numero_rate_versate=None):
        if not self.tipo_operazione == "S":
            return None

        if self.tipo_piano_to == 'PIC':
            if tipo_qta_movimento == 'I' and importo > 0 and not numero_rate_versate and not quote:
                self.importo = importo
                self.quote = 0
            elif tipo_qta_movimento == 'Q' and quote > 0 and not numero_rate_versate:
                self.importo = 0
                self.quote = quote
            elif tipo_qta_movimento == 'T' and not importo and not quote and not numero_rate_versate:
                self.importo = 0
                self.quote = 0
            else:
                return "ERRORE 1"  # FIXME: errore

        elif (
            self.tipo_piano_to == 'PAC' and importo > 0 and
            numero_rate_versate > 0 and tipo_qta_movimento == 'I' and not quote
        ):
            self.importo = importo
            self.numero_rate_versate = numero_rate_versate
        elif (
            self.tipo_piano_to == 'PAD' and importo > 0 and
            numero_rate_versate > 0 and tipo_qta_movimento == 'I' and not quote
        ):
            self.importo = importo
            self.numero_rate_versate = numero_rate_versate
        else:
            return "ERRORE 2"  # FIXME: errore

        self.status = 'FF'
        self.tipo_qta_movimento = tipo_qta_movimento
        self.iban = ''

    def set_valori_rimborso(self, tipo_qta_movimento, iban, importo=None, quote=None):
        if not self.tipo_operazione == "D":
            return None

        if tipo_qta_movimento == 'I' and importo > 0 and not quote:
            self.importo = importo
            self.quote = 0
        elif tipo_qta_movimento == 'Q' and quote > 0 and not importo:
            self.importo = 0
            self.quote = quote
        elif tipo_qta_movimento == 'T' and not importo and not quote:
            self.importo = 0
            self.quote = 0
        else:
            return "ERRORE"  # FIXME: errore

        self.status = 'FF'
        self.tipo_qta_movimento = tipo_qta_movimento
        self.iban = iban

    def can_operate(self):
        mifid = 'N'
        if self.mifid:
            mifid = 'S'

        if not hasattr(self, 'area_riservata'):
            self.set_areariservata()

        valore = self.importo
        if self.tipo_qta_movimento == "Q":
            valore = self.quote

        return self.area_riservata.op_online_check_can_operate(
            self.codice_fiscale, self.codice_cliente, self.get_tipo_operazione_display(),
            self.codice_rapporto_from, self.codice_deposito_from, self.codice_fondo_from,
            self.codice_tipocontratto_from,
            self.codice_rapporto_to, self.codice_deposito_to, self.codice_fondo_to, self.codice_tipocontratto_to,
            self.get_tipo_qta_movimento_display(), valore,
            self.numero_rate_versate, mifid, self.iban
        )

    def save_sottoscrizione(self):
        if not self.tipo_operazione == "I":
            return None

        if not self.can_operate()['OK']:
            return self.can_operate()

        mifid = 'N'
        if self.mifid:
            mifid = 'S'

        simulazione = 'S'
        if not self.simulazione:
            simulazione = 'N'

        if not hasattr(self, 'area_riservata'):
            self.set_areariservata()

        bic = ''
        result = self.area_riservata.op_online_save_sottoscrizione(
            self.codice_cliente, self.codice_rapporto_to, self.codice_deposito_from, self.codice_deposito_to,
            self.codice_fondo_to, self.codice_tipocontratto_to, self.cognome, self.nome, self.codice_fiscale,
            mifid, self.quote, self.importo, self.numero_rate_piano, self.importo_rata, self.numero_rate_versate,
            simulazione, self.iban, bic
        )
        if isinstance(result, long) and not isinstance(result, dict):
            # e' il numero dell'operazione
            self.status = "FN"
            self.data_operazione = datetime.datetime.now()
            self.numero_operazione = result
            # ritorno un dizionario
            return {'OK': True, 'numero_operazione': result}
        return result

    def save_switch(self):
        if not self.tipo_operazione == "S":
            return None

        if not self.can_operate()['OK']:
            return self.can_operate()

        mifid = 'N'
        if self.mifid:
            mifid = 'S'

        simulazione = 'S'
        if not self.simulazione:
            simulazione = 'N'

        switch_totale = 'N'
        if self.tipo_qta_movimento == 'T':
            switch_totale = 'S'

        if not hasattr(self, 'area_riservata'):
            self.set_areariservata()

        result = self.area_riservata.op_online_save_switch(
            self.codice_cliente, self.codice_rapporto_from, self.codice_deposito_from, self.codice_fondo_from,
            self.codice_tipocontratto_from, self.cognome, self.nome, self.codice_fiscale, switch_totale,
            mifid, self.quote, self.importo, self.numero_rate_piano, self.importo_rata, self.numero_rate_versate,
            simulazione, self.codice_rapporto_to, self.codice_deposito_to, self.codice_fondo_to,
            self.codice_tipocontratto_to
        )
        if isinstance(result, long) and not isinstance(result, dict):
            # e' il numero dell'operazione
            self.status = "FN"
            self.data_operazione = datetime.datetime.now()
            self.numero_operazione = result
            # ritorno un dizionario
            return {'OK': True, 'numero_operazione': result}
        return result

    def save_rimborso(self):
        if not self.tipo_operazione == "D":
            return None

        if not self.can_operate()['OK']:
            return self.can_operate()

        mifid = 'N'
        if self.mifid:
            mifid = 'S'

        simulazione = 'S'
        if not self.simulazione:
            simulazione = 'N'

        bic = ''
        rimborso_totale = 'N'
        if self.tipo_qta_movimento == 'T':
            rimborso_totale = 'S'

        if not hasattr(self, 'area_riservata'):
            self.set_areariservata()

        iban = self.iban.upper()

        result = self.area_riservata.op_online_save_rimborso(
            self.codice_cliente, self.codice_rapporto_from, self.codice_deposito_from, self.codice_fondo_from,
            self.codice_tipocontratto_from, self.cognome, self.nome, self.codice_fiscale, rimborso_totale,
            mifid, self.quote, self.importo, self.numero_rate_piano, self.importo_rata, self.numero_rate_versate,
            simulazione, iban, bic
        )

        if isinstance(result, long) and not isinstance(result, dict):
            # e' il numero dell'operazione
            self.status = "FN"
            self.data_operazione = datetime.datetime.now()
            self.numero_operazione = result
            # ritorno un dizionario
            return {'OK': True, 'numero_operazione': result}
        return result

    def __unicode__(self):
        return 'Operazione'

    class Meta:
        verbose_name = _('Operazione')
        verbose_name_plural = _('Operazioni')


class Applicazione(UserModel, DateModel, StatusModel, OrderedModel):
    codice = models.CharField(
        max_length=6,
        unique=True,
        choices=settings.AREARISERVATA_APPLICAZIONE_CODICE_CHOICES,
    )
    description = models.CharField(
        _("Description"),
        max_length=250,
    )
    # modal_page_slug = models.SlugField(
    #     _("Modal page slug"),
    #     max_length=100,
    #     null=True, blank=True,
    #     help_text=_("(Optional) url for remote modal info page")
    # )
    # sottotitolo_from = models.CharField(
    #     max_length=100,
    #     null=True, blank=True,
    #     help_text=_("(Optional) per inserire sottotitolo nella pagina APP From")
    # )

    class Meta(object):
        verbose_name = _("Applicazione")
        verbose_name_plural = _("Applicazioni")
        ordering = ('ordering',)

    def __unicode__(self):
        return 'Applicazione: %s' % self.get_codice_display()


class OperationSecurityCode(UserModel, DateModel, StatusModel, OrderedModel):
    customer = models.ForeignKey(
        User,
        related_name="operation_security_codes",
        verbose_name=_("customer"),
    )
    customer_code = models.IntegerField(
        verbose_name=_('customer code'),
    )
    customer_ssn = models.CharField(
        max_length=16,
        verbose_name=_('ssn'),
    )
    operation_number = models.IntegerField(
        unique=True,
        verbose_name=_('operation number'),
    )
    operation_name = models.CharField(
        max_length=16,
        choices=settings.AREARISERVATA_INVESTI_DISINVESTI_TIPO_OPERAZIONE_CHOICES,
        verbose_name=_('operation name'),
    )
    secure_code = models.CharField(
        max_length=6,
        verbose_name=_('secure code'),
    )
    promoter_code = models.CharField(
        max_length=12,
        verbose_name=_('promoter code'),
    )
    data = models.TextField(
        verbose_name=_('data')
    )

    class Meta:
        verbose_name = _('operation security code')
        verbose_name_plural = _('operation security codes')
        ordering = ('ordering', '-date_last_modify', 'operation_name')
        permissions = (
            ("list_operationsecuritycode", "Can list operation security code"),
            ("detail_operationsecuritycode", "Can detail operation security code"),
            ("disable_operationsecuritycode", "Can disable operation security code"),
        )

    def __unicode__(self):
        return u'{}-{}-{}'.format(
            self.operation_name, self.operation_number, self.customer_code
        )

    def save(self, *args, **kwargs):
        self.set_customer()
        self.set_secure_code()
        super(OperationSecurityCode, self).save(*args, **kwargs)

    def set_secure_code(self):
        if not self.id:
            if not self.secure_code:
                self.secure_code = self.generate_secure_code()

    def set_customer(self):
        if not self.id:
            if not hasattr(self, 'customer'):
                rac = RichiestaAttivazioneCliente.objects.get(codice_fiscale=self.customer_ssn)
                self.customer = rac.profilo.user

    def generate_secure_code(self, string_length=6):
        allowed_chars = ''.join((string.ascii_uppercase, string.digits))
        return ''.join(random.choice(allowed_chars) for _ in range(string_length))

    @property
    def uidb36(self):
        return int_to_base36(self.id)

    @property
    def code_for_token(self):
        """
        Returns code used for token
        """
        return '{}{}'.format(self.secure_code, self.operation_number)

    @property
    def token(self):
        """
        Returns token
        """
        token_generator = TokenGenerator()
        return token_generator.make_token(self.code_for_token)

    def check_token(self, token):
        """
        Verify the validity of token
        :param token: previously generated
        :return: token validity
        """
        token_generator = TokenGenerator()
        if not token_generator.check_token(self.code_for_token, token):
            return False
        return True

    def check_token_by_manual_secure_code(self, secure_code):
        """
        Verify the validity of token with manual secure code.
        Used by the acomea user to validate the transaction on behalf of the customer
        :param secure_code: manual code insert by acomea user
        :return: token validity
        """
        token_generator = TokenGenerator()
        code_to_verify = '{}{}'.format(secure_code, self.operation_number)
        if not token_generator.check_token(code_to_verify, self.token):
            return False
        return True


if settings.CMS_ENABLED:
    import acomea.areariservata.cms.plugins
    import acomea.areariservata.cms.app