#! -*- coding: utf-8 -*-
import locale
import logging
import datetime

from pysimplesoap.client import SoapClient

from django.conf import settings

log = logging.getLogger("acomea.areariservata")


# ecommerce
class AcomeaClientRiservataCms:
    """
    SOAP_AR: http://93.57.3.196:8282/service.asmx?wsdl
    """

    def __init__(self):
        log.debug("AcomeaClientRiservataCms init")
        try:
            self.client = SoapClient(wsdl=settings.SOAP_RESERVED_URL, trace=False)
        except Exception, exc:
            log.warning("AcomeaClientRiservataCms Exception %s" % str(exc))
            raise

    def is_area_active(self):
        """
        http://93.57.3.196:8282/service.asmx?op=AreaRiservata_IsActive
        """
        data = self.client.AreaRiservata_IsActive()
        return data['AreaRiservata_IsActiveResult']

    def op_online_is_rimborso_active(self):
        """
        http://93.57.3.196:8282/service.asmx?op=AreaRiservata_IsActive_OpOnlineRimborso
        """
        return self.client.AreaRiservata_IsActive_OpOnlineRimborso()['AreaRiservata_IsActive_OpOnlineRimborsoResult']

    def op_online_is_sottoscrizione_active(self):
        """
        http://93.57.3.196:8282/service.asmx?op=AreaRiservata_IsActive_OpOnlineSott
        """
        return self.client.AreaRiservata_IsActive_OpOnlineSott()['AreaRiservata_IsActive_OpOnlineSottResult']

    def op_online_is_switch_active(self):
        """
        http://93.57.3.196:8282/service.asmx?op=AreaRiservata_IsActive_OpOnlineSwitch
        """
        return self.client.AreaRiservata_IsActive_OpOnlineSwitch()['AreaRiservata_IsActive_OpOnlineSwitchResult']

    def get_cliente(self, cod_fiscale):
        """
        Torna una struttura coi dati principali di un cliente dato il suo codice fiscale,
        o None se il cliente non esiste.
        """
        result = self.client.GetCliente(cod_fiscale)
        result_key = result.keys()[0]
        data = self.client.GetCliente(cod_fiscale)[result_key]
        if data['CODFISC']:
            return data
        else:
            return None

    def get_cliente_sgr(self, cod_fiscale):
        """
        Torna una struttura coi dati principali di un cliente dato il suo codice fiscale,
        o None se il cliente non esiste.
        Sostituisce il precedente che resituiva NULL se il cliente aveva solo Gestioni

        http://93.57.3.196:8282/service.asmx?op=GetClienteSGR

        SEND
        <cf>string</cf>

        RETURN
        <IsDead>boolean</IsDead>
        <IsPF>boolean</IsPF>
        <Found>boolean</Found>
        <DtNasc>string</DtNasc>
        <CodCliFC>string</CodCliFC>
        <CodCliGI>string</CodCliGI>
        <Nome>string</Nome>
        <Cognome>string</Cognome>
        <Mail>string</Mail>
        <CODFISC>string</CODFISC>
        <TipoPartecipante>string</TipoPartecipante>
        """
        data = self.client.GetClienteSGR(cod_fiscale)['GetClienteSGRResult']
        if data['CODFISC']:
            return data
        else:
            return None

    def op_online_datagrid(self, cod_cli, tipo_partecipazione):
        """
        http://93.57.3.196:8282/service.asmx?op=OpOnline_DataGrid
        <CodCliente>int</CodCliente>
        <tp>INTESTATARIO or COINTESTATARIO or DELEGATO</tp>
        quali tipi di partecipazioni devo visualizzare dipendono da get_cliente
        """
        posizioni = self.client.OpOnline_DataGrid(cod_cli, tipo_partecipazione)['OpOnline_DataGridResult']
        lista_posizioni = []
        for posizione in posizioni:
            lista_posizioni.append(posizione['PosizioniSource'])
        return lista_posizioni

    def qst_must_compile_ar_by_rapp(self, cod_fiscale, cod_rapporto, cod_fondo):
        """
        http://93.57.3.196:8282/service.asmx?op=QST_MustCompileARByRap
        <cf>string</cf>
        <cod_rapp>long</cod_rapp>
        <cod_fon>string</cod_fon>
        """
        return self.client.QST_MustCompileARByRap(cod_fiscale, cod_rapporto, cod_fondo)['QST_MustCompileARByRapResult']

    def qst_must_compile_ar_by_sott(self, cod_fiscale):
        """
        http://93.57.3.196:8282/service.asmx?op=QST_MustCompileARBySot
        <cf>string</cf>
        All'attivazione del profilo dispositivo occorre chiamare questo metodo
        """
        return self.client.QST_MustCompileARBySot(cod_fiscale)['QST_MustCompileARBySotResult']

    def qst_is_compiled(self, questionario, cod_fiscale):
        """
        http://93.57.3.196:8282/service.asmx?op=QST_IsCompiled
        <_tq>ANTIRICICLAGGIO or APPROPRIATEZZA or ADEGUATEZZA</_tq>
        <_cf>string</_cf>
        """
        return self.client.QST_IsCompiled(questionario, cod_fiscale)['QST_IsCompiledResult']

    def qst_is_compiled_reset(self, questionario, cod_fiscale):
        """
        http://93.57.3.196:8282/service.asmx?op=QST_IsCompiledReset
        Resetta questionari, utilizzabile sui seguenti codici fiscali:
        NGRNRE21A66G337Q;SLLGPP36C30F151F;CTLPQL54B27E716D;MRNLRD64H16G843J;
        CCCPLA73M03C933F;LDAGPP39P11F205X;VTLFBA66P10F205J;CLPMPA31R11L433I
        """
        return self.client.QST_IsCompiledReset(questionario, cod_fiscale)

    def op_online_check_appropriatezza(self, cod_fiscale, cod_fondo):
        """
        http://93.57.3.196:8282/service.asmx?op=OpOnline_CheckAppropriatezza
        <cf>string</cf>
        <cod_fon>string</cod_fon>
        """
        return self.client.OpOnline_CheckAppropriatezza(cod_fiscale, cod_fondo)['OpOnline_CheckAppropriatezzaResult']

    def op_online_get_elenco_sottoscrizioni_aggiuntive(self, cod_cli, cod_rapp, cod_dep, cod_fon, cod_tc):
        """
        http://93.57.3.196:8282/service.asmx?op=OpOnline_GetElencoSottAgg
        <codCliente>int</codCliente>
        <codRapp>int</codRapp>
        <coddep>string</coddep>
        <CodFon>string</CodFon>
        <CodTC>string</CodTC>
        <EX>int</EX>
        Restituisce l'elenco dei fondi su cui effettuare investimento.
        EX = 1: Fondi già presenti nel prorio rapporto
        EX = 0: Fondi disponibili ma non presenti nel proprio rapporto
        """
        posizioni = self.client.OpOnline_GetElencoSottAgg(cod_cli, cod_rapp, cod_dep, cod_fon, cod_tc)[
            'OpOnline_GetElencoSottAggResult']
        lista_posizioni = []
        for posizione in posizioni:
            lista_posizioni.append(posizione['PosizioniTarget'])
        return lista_posizioni

    def op_online_get_elenco_switch_aggiuntive(self, cod_cli, cod_rapp, cod_dep, cod_fon, cod_tc):
        """
        http://93.57.3.196:8282/service.asmx?op=OpOnline_GetElencoSwitch
        <codCliente>int</codCliente>
        <codRapp>int</codRapp>
        <coddep>string</coddep>
        <CodFon>string</CodFon>
        <CodTC>string</CodTC>
        <EX>int</EX>
        Restituisce l'elenco dei fondi su cui effettuare lo switch.
        EX = 1: Fondi già presenti nel prorio rapporto
        EX = 0: Fondi disponibili ma non presenti nel proprio rapporto
        """
        posizioni = self.client.OpOnline_GetElencoSwitch(cod_cli, cod_rapp, cod_dep, cod_fon, cod_tc)[
            'OpOnline_GetElencoSwitchResult']
        lista_posizioni = []
        for posizione in posizioni:
            lista_posizioni.append(posizione['PosizioniTarget'])
        return lista_posizioni

    def op_online_check_can_operate(
            self, cod_fiscale, cod_cli, tipo_operazione, cod_rapp_from, cod_dep_from, cod_fon_from, tipo_contratto_from,
            cod_rapp_to, cod_dep_to, cod_fon_to, tipo_contratto_to, tipo_qta_movimento, valore_qta_movimento,
            num_rate_versate,
            check_manleva, iban
    ):
        """
        http://93.57.3.196:8282/service.asmx?op=OpOnline_CheckCanOperate
        <cf>string</cf>
        <c_sot>long</c_sot>
        <tipoOperazione>SOTTOSCRIZIONE or SWITCH or RIMBORSO</tipoOperazione>
        <codRappFrom>string</codRappFrom>
        <codDepFrom>string</codDepFrom>
        <codFonFrom>string</codFonFrom>
        <tcFrom>string</tcFrom>
        <codRappTo>string</codRappTo>
        <codDepTo>string</codDepTo>
        <codFonTo>string</codFonTo>
        <tcTo>string</tcTo>
        <TipoQtaMovimento>QUOTE or IMPORTO or TOTALE</TipoQtaMovimento>
        <ValoreQtaMovimento>decimal</ValoreQtaMovimento>
        <NumRateVersate>long</NumRateVersate>
        <CheckManLeva>string</CheckManLeva>
        <IBAN>string</IBAN>
        """
        return self.client.OpOnline_CheckCanOperate(
            cod_fiscale, cod_cli, tipo_operazione, cod_rapp_from, cod_dep_from, cod_fon_from, tipo_contratto_from,
            cod_rapp_to, cod_dep_to, cod_fon_to, tipo_contratto_to, tipo_qta_movimento, valore_qta_movimento,
            num_rate_versate,
            check_manleva, iban
        )['OpOnline_CheckCanOperateResult']

    def op_online_save_sottoscrizione(
            self, cod_cli, cod_rapp_to, cod_dep_from, cod_dep_to, cod_fon_to,
            cod_tipocontratto_to, cognome, nome, cod_fisc, mifid, quote, importo, num_rate_piano, importo_rata,
            numero_rate_versate, simulazione, iban, bic
    ):
        """
        http://93.57.3.196:8282/service.asmx?op=OpOnline_AddSottAdd2
        <codcli>long</codcli>
        <codrapp>long</codrapp>
        <coddepFrom>string</coddepFrom>
        <coddepTo>string</coddepTo>
        <codfon>string</codfon>
        <tipocontratto>string</tipocontratto>
        <cognome>string</cognome>
        <nome>string</nome>
        <codfisc>string</codfisc>
        <dtnascita>string</dtnascita>
        <mifid>string</mifid>
        <quote>decimal</quote>
        <importo>decimal</importo>
        <numratepiano>short</numratepiano>
        <importorata>decimal</importorata>
        <numrateversate>short</numrateversate>
        <simulazione>string</simulazione>
        <IBAN>string</IBAN>
        <BIC>string</BIC>
        """
        return self.client.OpOnline_AddSottAdd2(
            cod_cli, cod_rapp_to, cod_dep_from, cod_dep_to, cod_fon_to, cod_tipocontratto_to, cognome, nome, cod_fisc,
            mifid, quote, importo, num_rate_piano, importo_rata, numero_rate_versate, simulazione, iban, bic
        )['OpOnline_AddSottAdd2Result']

    def op_online_save_switch(
            self, cod_cli, cod_rapp_from, cod_dep_from, cod_fon_from,
            cod_tipocontratto_from, cognome, nome, cod_fisc, switch_totale, mifid, quote, importo, num_rate_piano,
            importo_rata,
            numero_rate_versate, simulazione, cod_rapp_to, cod_dep_to, cod_fon_to, cod_tipocontratto_to
    ):
        """
        http://93.57.3.196:8282/service.asmx?op=OpOnline_ADDSwitch2
        <codcli>long</codcli>
        <codrapp>long</codrapp>
        <coddep>string</coddep>
        <codfon>string</codfon>
        <tipocontratto>string</tipocontratto>
        <cognome>string</cognome>
        <nome>string</nome>
        <codfisc>string</codfisc>
        <switchtotale>string</switchtotale>
        <mifid>string</mifid>
        <quote>decimal</quote>
        <importo>decimal</importo>
        <numratepiano>short</numratepiano>
        <importorata>decimal</importorata>
        <numrateversate>short</numrateversate>
        <simulazione>string</simulazione>
        <switchcodrapp>long</switchcodrapp>
        <switchcoddep>string</switchcoddep>
        <switchcodfon>string</switchcodfon>
        <switchtc>string</switchtc>
        """
        return self.client.OpOnline_ADDSwitch2(
            cod_cli, cod_rapp_from, cod_dep_from, cod_fon_from,
            cod_tipocontratto_from, cognome, nome, cod_fisc, switch_totale, mifid, quote, importo, num_rate_piano,
            importo_rata,
            numero_rate_versate, simulazione, cod_rapp_to, cod_dep_to, cod_fon_to, cod_tipocontratto_to
        )['OpOnline_ADDSwitch2Result']

    def op_online_save_rimborso(
            self, cod_cli, cod_rapp_from, cod_dep_from, cod_fon_from,
            cod_tipocontratto_from, cognome, nome, cod_fisc, rimborso_totale,
            mifid, quote, importo, num_rate_piano, importo_rata,
            numero_rate_versate, simulazione, iban, bic
    ):
        """
        http://93.57.3.196:8282/service.asmx?op=OpOnline_AddRimborso2
        <codcli>long</codcli>
        <codrapp>long</codrapp>
        <coddep>string</coddep>
        <codfon>string</codfon>
        <tipocontratto>string</tipocontratto>
        <cognome>string</cognome>
        <nome>string</nome>
        <codfisc>string</codfisc>
        <rimborsototale>string</rimborsototale>
        <mifid>string</mifid>
        <quote>decimal</quote>
        <importo>decimal</importo>
        <numratepiano>short</numratepiano>
        <importorata>decimal</importorata>
        <numrateversate>short</numrateversate>
        <simulazione>string</simulazione>
        <IBAN>string</IBAN>
        <BIC>string</BIC>
        """
        return self.client.OpOnline_AddRimborso2(
            cod_cli, cod_rapp_from, cod_dep_from, cod_fon_from,
            cod_tipocontratto_from, cognome, nome, cod_fisc, rimborso_totale,
            mifid, quote, importo, num_rate_piano, importo_rata,
            numero_rate_versate, simulazione, iban, bic
        )['OpOnline_AddRimborso2Result']

    def check_cliente(self, cod_cliente, cod_fiscale, data_nascita):
        """
        Controlla se un cliente esistente è attivo.
        """
        data = self.client.CheckCliente(cod_cliente, cod_fiscale, _format_data(data_nascita))
        return data['CheckClienteResult']

    def get_riepilogo(self, prodotto, partecipazione, cod_fiscale):
        """
        Torna tre valori aggregati per un certo prodotto / tipo di partecipazione di un cliente,
        dato dal suo codice fiscale
        """
        prodotto = prodotto.upper()
        partecipazione = partecipazione.upper()
        riepilogo = self.client.GetRiepilogo(prodotto, partecipazione, cod_fiscale)['GetRiepilogoResult']
        output = {}
        try:
            locale.setlocale(locale.LC_ALL, 'it_IT.utf8')
        except:
            locale.setlocale(locale.LC_ALL, 'it_IT.UTF-8')
        output['valore_patrimonio'] = locale.format("%.2f", riepilogo['ValATT'], grouping=True, monetary=True)
        output['saldo_sottoscrizioni'] = locale.format("%.2f", riepilogo['SaldoSR'], grouping=True, monetary=True)
        output['utile_perdita'] = locale.format("%.2f", riepilogo['ProfitLoss'], grouping=True, monetary=True)
        output['data_posizione'] = riepilogo['DataPosizione']
        output['codice_cliente'] = riepilogo['Codice_Cliente']
        return output

    def get_dettaglio(self, prodotto, partecipazione, cod_fiscale):
        """
        Torna una struttura dati col dettaglio per un certo prodotto / tipo di partecipazione di un cliente,
        dato dal suo codice fiscale
        """
        prodotto = prodotto.upper()
        partecipazione = partecipazione.upper()
        dettaglio = self.client.GetSintesiRapporti(prodotto, partecipazione, cod_fiscale)['GetSintesiRapportiResult']
        output = []
        for array_sr in dettaglio:
            sr = array_sr['SintesiRapporti']
            linea = {}
            height = 30
            linea['valori_patrimoniali'] = locale.format("%.2f", sr['ValATT'], grouping=True, monetary=True)
            linea['tipo_partecipazione'] = sr['TipoPartecipazione']
            linea['descrizione_tipo_partecipazione'] = sr['TipoPartecipazioneDescr']
            linea['tipo_prodotto'] = sr['TipoProdotto']
            linea['descrizione_tipo_prodotto'] = sr['TipoProdottoDescr']
            linea['cod_prodotto'] = sr['CodProdotto']
            linea['descrizione_prodotto'] = sr['DescrProdotto']
            linea['quota'] = sr['Quota']
            linea['quote'] = sr['Quote']
            linea['classe_prodotto'] = None
            try:
                linea['classe_prodotto'] = sr['DescrProdotto'].replace('CLASSE', 'CL').split('CL')[-1].strip().lower()
            except:
                pass
            linea['saldo'] = locale.format("%.2f", sr['SaldoSR'], grouping=True, monetary=True)
            linea['contratto'] = sr['Contratto']
            linea['utili'] = locale.format("%.2f", sr['ProfitLoss'], grouping=True, monetary=True)
            linea['app'] = sr['App']
            linea['rapporto'] = sr['Rapporto']

            # for l in array_sr:
            #     rapporto = l.Rapporto
            #     contratto = l.Contratto
            #     if len(l.DescrProdotto) > 26:
            #         height += 40
            #     else:
            #         height += 30

            output.append(linea)
        return output

    def get_operazioni_in_corso_web(self, cod_fiscale):
        """
        Restituisce le operazioni in corso inserite tramite portale non ancora valorizzate
        http://93.57.3.196:8282/service.asmx?op=OpOnline_GetOpInCorsoWEB
        <cf>string</cf>
        """
        operazioni = self.client.OpOnline_GetOpInCorsoWEB(cod_fiscale)['OpOnline_GetOpInCorsoWEBResult']
        lista_operazioni = []
        for operazione in operazioni:
            lista_operazioni.append(operazione['OperazioneWEB'])
        return lista_operazioni

    def get_operazioni_in_corso_contra(self, cod_fiscale):
        """
        Restituisce le operazioni in corso inserite in AS400 non ancora valorizzate
        http://93.57.3.196:8282/service.asmx?op=OpOnline_GetOpInCorsoCONTRA
        <cf>string</cf>
        """
        operazioni = self.client.OpOnline_GetOpInCorsoCONTRA(cod_fiscale)['OpOnline_GetOpInCorsoCONTRAResult']
        lista_operazioni = []
        for operazione in operazioni:
            lista_operazioni.append(operazione['DepositoMovimento'])
        return lista_operazioni

    def get_operazioni_effettuate(self, cod_fiscale, tipo_partecipazione='INTESTATARIO', date_begin=None,
                                  date_end=None):
        """
        Restituisce le operazioni effettuate valorizzate
        http://93.57.3.196:8282/service.asmx?op=CreateElencoMovimenti
        <_cf>string</_cf>
        <_TipoPartecipazione>INTESTATARIO or COINTESTATARIO or DELEGATO</_TipoPartecipazione>
        <_dt1>dateTime</_dt1>
        <_dt2>dateTime</_dt2>
        """
        if not date_begin:
            date_begin = datetime.date(1970, 1, 1)
        if not date_end:
            date_end = datetime.date.today()

        operazioni = self.client.CreateElencoMovimenti(cod_fiscale, tipo_partecipazione, date_begin, date_end)[
            'CreateElencoMovimentiResult']
        lista_operazioni = []
        for operazione in operazioni:
            lista_operazioni.append(operazione['DepositoMovimento'])
        return lista_operazioni

    def annulla_operazione_in_corso_web(self, numero_operazione, cod_fiscale):
        """
        Annulla l'operazione in corso web
        http://93.57.3.196:8282/service.asmx?op=AnnullaOperazioneWEB
        <_nop>long</_nop>
        <_cf>string</_cf>
        """
        result = self.client.AnnullaOperazioneWEB(numero_operazione, cod_fiscale)['AnnullaOperazioneWEBResult']
        return result


class AcomeaG5:
    """
    Client più lento, ma che non da problemi usato da cms
    """

    def __init__(self, codice_fiscale):
        log.debug("AcomeaG5 init")
        try:
            self.client = SoapClient(wsdl=settings.SOAP_G5_URL, trace=False)
            self.codice_fiscale = codice_fiscale.upper()
            # FONDI COMUNI
            self.get_clienti = self.client.CheckSingle(self.codice_fiscale)['CheckSingleResult']
            # GESTIONI INDIVISUALI
            self.get_clienti_gi = self.client.CheckSingleGI(self.codice_fiscale)['CheckSingleGIResult']

        except Exception, exc:
            log.warning("AcomeaClientCms Exception %s" % str(exc))
            raise

    def is_cliente(self):
        """
        Restituisce True se è cliente o False se non lo è
        """
        data = self.get_clienti
        return data['ExistSquadra']

    def is_cliente_gi(self):
        """
        Restituisce True se è cliente o False se non lo è
        """
        data = self.get_clienti_gi
        return data['ExistSquadra']

    def is_cliente_sgr(self):
        return (self.is_cliente(), self.is_cliente_gi())

    def can_be_disposing(self):
        """
        Restituisce True se il cliente e' di una rete per la
        quale puo' essere dispositivo
        """
        data = self.get_clienti
        value = data.get('CanBeDisposing', None)
        return True if value else False

    def get_anagrafiche(self):
        """
        Restituisce la lista delle anagrafiche
        """
        data = self.get_clienti
        lista_anagrafiche = []
        for anagrafica in data['Anagrafiche']:
            lista_anagrafiche.append(anagrafica.values()[0])
        return lista_anagrafiche

    def get_anagrafiche_gi(self):
        """
        Restituisce la lista delle anagrafiche
        """
        data = self.get_clienti_gi
        lista_anagrafiche = []
        for anagrafica in data['Anagrafiche']:
            lista_anagrafiche.append(anagrafica.values()[0])
        return lista_anagrafiche

    def get_rapporti(self):
        """
        Restituisce la lista dei rapporti
        """
        data = self.get_clienti
        lista_rapporti = []
        for rapporto in data['Rapporti']:
            lista_rapporti.append(rapporto.values()[0])
        return lista_rapporti

    def get_rapporti_gi(self):
        """
        Restituisce la lista dei rapporti
        """
        data = self.get_clienti_gi
        lista_rapporti = []
        for rapporto in data['Rapporti']:
            lista_rapporti.append(rapporto.values()[0])
        return lista_rapporti

    def get_squadre(self):
        """
        Restituisce la lista delle squadre
        """
        data = self.get_clienti
        lista_squadre = []
        for squadra in data['Squadre']:
            lista_squadre.append(squadra.values()[0])
        return lista_squadre

    def get_squadre_gi(self):
        """
        Restituisce la lista delle squadre
        """
        data = self.get_clienti_gi
        lista_squadre = []
        for squadra in data['Squadre']:
            lista_squadre.append(squadra.values()[0])
        return lista_squadre

    def get_classe(self):
        """
        Restituisce la Classe A1 o A2
        """
        data = self.get_clienti
        return data['Classe']

    def get_classe_gi(self):
        """
        Restituisce la Classe A1 o A2
        """
        data = self.get_clienti_gi
        return data['Classe']

    def get_rete(self):
        """
        Restituisce la Rete Fondi Comuni
        """
        data = self.get_clienti
        return {'rete': data['Rete'], 'descrizione_rete': data['DescrizioneRete']}

    def get_rete_gi(self):
        """
        Restituisce la Rete Gestioni Individuali
        """
        data = self.get_clienti_gi
        return {'rete': data['Rete'], 'descrizione_rete': data['DescrizioneRete']}

    def get_reti(self):
        """
        Restituisce Lista tutte le reti
        """
        return [self.get_rete(), self.get_rete_gi()]

    def get_anagrafica(self):
        """
        Restituisce l'anagrafica del codice fiscale
        """
        for anagrafica in self.get_anagrafiche():
            if anagrafica['CodiceFiscale'].upper().replace(' ', '') == self.codice_fiscale:
                return anagrafica

        for anagrafica in self.get_anagrafiche_gi():
            if anagrafica['CodiceFiscale'].upper().replace(' ', '') == self.codice_fiscale:
                return anagrafica

    def get_all_reti_descriptions(self):
        reti = []
        for rapporti in (self.get_clienti.get('Rapporti'), self.get_clienti_gi.get('Rapporti'),):
            for rapporto in self.get_clienti.get('Rapporti'):
                desc_rete = rapporto.get('Rapporto').get('DescrizioneRete')
                if desc_rete and not desc_rete.strip() in reti:
                    reti.append(rapporto.get('Rapporto').get('DescrizioneRete').strip())
        return reti

    def get_all_classi(self):
        """
        ritorna tutte le classi presenti in get_rapporti() e get_rapporti_gi()
        """
        classi = set()
        for rapporto in self.get_rapporti() + self.get_rapporti_gi():
            if 'Classe' in rapporto and not rapporto['Classe'] is None:
                classi.add(rapporto['Classe'])
        return list(classi)

    def get_indirizzo_residenza(self):
        """
        Restituisce l'indirizzo di residenza del codice fiscale
        """
        anagrafica = self.get_anagrafica()
        indirizzo_residenza = {
            'Cap': anagrafica['CapResidenza'],
            'Citta': anagrafica['CittaResidenza'],
            'Indirizzo': anagrafica['IndirizzoResidenza'],
            'Localita': anagrafica['LocalitaResidenza'],
            'Nominativo': None,
            'Provincia': anagrafica['ProvinciaResidenza'],
            'Stato': anagrafica['StatoResidenza'],
            'StatoDescr': anagrafica['StatoResidenzaDescr']
        }
        return indirizzo_residenza

    def get_indirizzi_rapporti(self):
        """
        Restituisce la lista defli indirizzi dei rapporti
        """
        lista_indirizzi = []
        for rapporto in self.get_rapporti():
            if (
                'IndirizzoRecapito' in rapporto
                and 'Indirizzo' in rapporto['IndirizzoRecapito']
                and rapporto['IndirizzoRecapito']['Indirizzo']
            ):
                lista_indirizzi.append(rapporto.get('IndirizzoRecapito'))
        return lista_indirizzi

    def get_indirizzi_rapporti_gi(self):
        """
        Restituisce la lista defli indirizzi dei rapporti
        """
        lista_indirizzi = []
        for rapporto in self.get_rapporti_gi():
            if (
                'IndirizzoRecapito' in rapporto
                and 'Indirizzo' in rapporto['IndirizzoRecapito']
                and rapporto['IndirizzoRecapito']['Indirizzo']
            ):
                lista_indirizzi.append(rapporto.get('IndirizzoRecapito'))
        return lista_indirizzi

    def get_indirizzi(self):
        return [self.get_indirizzo_residenza(), ] + self.get_indirizzi_rapporti() + self.get_indirizzi_rapporti_gi()
